import os
from datetime import datetime
from typing import Optional, Type, Callable

from cpl_core.application import StartupExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC

from bot_core.configuration.bot_logging_settings import BotLoggingSettings


class StartupSettingsExtension(StartupExtensionABC):
    def __init__(self):
        self._start_time = datetime.now()

    def configure_configuration(self, configuration: ConfigurationABC, environment: ApplicationEnvironmentABC):
        # this shit has to be done here because we need settings in subsequent startup extensions
        environment.set_working_directory(os.path.dirname(os.path.realpath(__file__)))
        configuration.add_environment_variables("SDB_")
        configuration.add_environment_variables("DISCORD_")

        configuration.add_json_file(f"config/appsettings.json", optional=False)
        configuration.add_json_file(f"config/appsettings.{environment.environment_name}.json", optional=True)
        configuration.add_json_file(f"config/appsettings.{environment.host_name}.json", optional=True)
        # load feature-flags
        configuration.add_json_file(f"config/feature-flags.json", optional=False)
        configuration.add_json_file(f"config/feature-flags.{environment.environment_name}.json", optional=True)
        configuration.add_json_file(f"config/feature-flags.{environment.host_name}.json", optional=True)

        configuration.add_configuration("Startup_StartTime", str(self._start_time))
        self._configure_settings_with_sub_settings(
            configuration, BotLoggingSettings, lambda x: x.files, lambda x: x.key
        )

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        pass

    @staticmethod
    def _configure_settings_with_sub_settings(
        config: ConfigurationABC, settings_type: Type, list_atr: Callable, atr: Callable
    ):
        settings: Optional[settings_type] = config.get_configuration(settings_type)
        if settings is None:
            return

        for sub_settings in list_atr(settings):
            config.add_configuration(f"{type(sub_settings).__name__}_{atr(sub_settings)}", sub_settings)
