import json

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.server_config_repository_abc import ServerConfigRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.server_afk_channel_ids_config import ServerAFKChannelIdsConfig
from bot_data.model.server_config import ServerConfig
from bot_data.model.server_team_role_ids_config import ServerTeamRoleIdsConfig
from bot_data.model.team_member_type_enum import TeamMemberTypeEnum


class ServerConfigRepositoryService(ServerConfigRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
    ):
        ServerConfigRepositoryABC.__init__(self)

        self._logger = logger
        self._context = db_context
        self._servers = servers

    def _get_team_role_ids(self, server_id: int) -> List[ServerTeamRoleIdsConfig]:
        ids = List(ServerTeamRoleIdsConfig)
        self._logger.trace(
            __name__,
            f"Send SQL command: {ServerTeamRoleIdsConfig.get_select_by_server_id_string(server_id)}",
        )
        results = self._context.select(ServerTeamRoleIdsConfig.get_select_by_server_id_string(server_id))
        for result in results:
            self._logger.trace(__name__, f"Got ServerTeamRoleIdsConfig with id {result[0]}")
            ids.append(
                ServerTeamRoleIdsConfig(
                    result[1],
                    TeamMemberTypeEnum(result[2]),
                    result[3],
                    result[4],
                    result[5],
                    id=result[0],
                )
            )

        return ids

    def _get_afk_channel_ids(self, server_id: int) -> List[int]:
        urls = List(int)
        self._logger.trace(
            __name__,
            f"Send SQL command: {ServerAFKChannelIdsConfig.get_select_by_server_id_string(server_id)}",
        )
        results = self._context.select(ServerAFKChannelIdsConfig.get_select_by_server_id_string(server_id))
        for result in results:
            self._logger.trace(__name__, f"Got ServerAFKChannelIdsConfig with id {result[0]}")
            urls.append(result[1])

        return urls

    def _from_result(self, result: tuple) -> ServerConfig:
        return ServerConfig(
            result[1],
            result[2],
            result[3],
            result[4],
            result[5],
            result[6],
            result[7],
            result[8],
            result[9],
            result[10],
            result[11],
            result[12],
            result[13],
            result[14],
            result[15],
            result[16],
            result[17],
            json.loads(result[18]),
            self._servers.get_server_by_id(result[19]),
            self._get_afk_channel_ids(result[19]),
            self._get_team_role_ids(result[19]),
            result[20],
            result[21],
            id=result[0],
        )

    def does_server_config_exists(self, server_id: int) -> bool:
        self._logger.trace(
            __name__,
            f"Send SQL command: {ServerConfig.get_select_by_server_id_string(server_id)}",
        )
        result = self._context.select(ServerConfig.get_select_by_server_id_string(server_id))

        return len(result) > 0

    def get_server_config_by_server(self, server_id: int) -> ServerConfig:
        self._logger.trace(
            __name__,
            f"Send SQL command: {ServerConfig.get_select_by_server_id_string(server_id)}",
        )
        result = self._context.select(ServerConfig.get_select_by_server_id_string(server_id))[0]

        return self._from_result(result)

    def get_server_config_by_id(self, config_id: int) -> ServerConfig:
        self._logger.trace(
            __name__,
            f"Send SQL command: {ServerConfig.get_select_by_id_string(config_id)}",
        )
        result = self._context.select(ServerConfig.get_select_by_id_string(config_id))[0]

        return self._from_result(result)

    def add_server_config(self, server_config: ServerConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_config.insert_string}")
        self._context.cursor.execute(server_config.insert_string)

    def update_server_config(self, server_config: ServerConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_config.udpate_string}")
        self._context.cursor.execute(server_config.udpate_string)

    def delete_server_config(self, server_config: ServerConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_config.delete_string}")
        self._context.cursor.execute(server_config.delete_string)

    def add_server_team_role_id_config(self, server_team_role_id: ServerTeamRoleIdsConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_team_role_id.insert_string}")
        self._context.cursor.execute(server_team_role_id.insert_string)

    def update_server_team_role_id_config(self, server_team_role_id: ServerTeamRoleIdsConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_team_role_id.udpate_string}")
        self._context.cursor.execute(server_team_role_id.udpate_string)

    def delete_server_team_role_id_config(self, server_team_role_id: ServerTeamRoleIdsConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_team_role_id.delete_string}")
        self._context.cursor.execute(server_team_role_id.delete_string)

    def add_server_afk_channel_config(self, server_afk_channel: ServerAFKChannelIdsConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_afk_channel.insert_string}")
        self._context.cursor.execute(server_afk_channel.insert_string)

    def update_server_afk_channel_config(self, server_afk_channel: ServerAFKChannelIdsConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_afk_channel.udpate_string}")
        self._context.cursor.execute(server_afk_channel.udpate_string)

    def delete_server_afk_channel_config(self, server_afk_channel: ServerAFKChannelIdsConfig):
        self._logger.trace(__name__, f"Send SQL command: {server_afk_channel.delete_string}")
        self._context.cursor.execute(server_afk_channel.delete_string)
