import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnVoiceStateUpdateABC
from cpl_translation import TranslatePipe

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.event_checks import EventChecks
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class BaseOnVoiceStateUpdateEventHelpChannel(OnVoiceStateUpdateABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        servers: ServerRepositoryABC,
        permissions: PermissionServiceABC,
        message_service: MessageServiceABC,
        t: TranslatePipe,
    ):
        OnVoiceStateUpdateABC.__init__(self)
        self._config = config
        self._logger = logger
        self._servers = servers
        self._permissions = permissions
        self._message_service = message_service
        self._t = t

        self._logger.info(__name__, f"Module {type(self)} loaded")

    async def _notify_team(self, member: discord.Member):
        self._logger.debug(__name__, f"Notify team that a member needs help")
        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{member.guild.id}")
        channel = member.guild.get_channel(settings.team_channel_id)
        await self._message_service.send_channel_message(
            channel,
            self._t.transform("modules.base.member_joined_help_voice_channel").format(member.mention),
            is_persistent=True,
        )
        self._logger.trace(__name__, f"Notified team that a member need help")

    @EventChecks.check_is_ready()
    async def on_voice_state_update(
        self,
        member: discord.Member,
        before: discord.VoiceState,
        after: discord.VoiceState,
    ):
        self._logger.debug(__name__, f"Module {type(self)} started")
        server = self._servers.get_server_by_discord_id(member.guild.id)
        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{server.discord_id}")
        if after.channel is None or after.channel.id != settings.help_voice_channel_id:
            return

        await self._notify_team(member)

        self._logger.debug(__name__, f"Module {type(self)} stopped")
