import discord
from cpl_core.configuration import ConfigurationABC
from cpl_discord.events import OnMessageABC

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.event_checks import EventChecks
from bot_core.logging.message_logger import MessageLogger
from bot_data.model.server_config import ServerConfig
from modules.level.service.level_service import LevelService


class LevelOnMessageEvent(OnMessageABC):
    def __init__(self, config: ConfigurationABC, logger: MessageLogger, level: LevelService):
        OnMessageABC.__init__(self)
        self._config = config
        self._logger = logger
        self._level = level

    @EventChecks.check_is_ready()
    async def on_message(self, message: discord.Message):
        self._logger.debug(__name__, f"Module {type(self)} started")
        if message.guild is None:
            return

        if message.author.bot:
            return

        server_config: ServerConfig = self._config.get_configuration(f"ServerConfig_{message.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(server_config.feature_flags, FeatureFlagsEnum.level_module):
            return

        try:
            await self._level.check_level(message.author)
        except Exception as e:
            self._logger.error(__name__, f"Level check by message failed", e)
